@@ -1,5 +1,5 @@
 # encoding=UTF-8
-# Copyright © 2008, 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008, 2009, 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,22 @@ import threading
 
 from djvu.sexpr import Expression, Symbol
 
-try:
-    DJVULIBRE_BIN_PATH = os.path.join(pkgconfig.Package('ddjvuapi').variable('exec_prefix'), 'bin')
-except (IOError, OSError):
-    DJVULIBRE_BIN_PATH = None
-    DJVUSED_PATH = 'djvused'
+djvused_path = None
+if os.name =='nt':
+    from . import dependencies
+    djvused_path = os.path.join(dependencies.djvulibre_path, 'djvused.exe')
 else:
-    DJVUSED_PATH = os.path.join(DJVULIBRE_BIN_PATH, 'djvused')
+    try:
+        djvulibre_bin_path = os.path.join(pkgconfig.Package('ddjvuapi').variable('exec_prefix'), 'bin')
+    except (IOError, OSError):
+        pass
+    else:
+        djvused_path = os.path.join(djvulibre_bin_path, 'djvused')
+if djvused_path is None or not os.path.isfile(djvused_path):
+    # Let's hope it's within $PATH...
+    djvused_path = 'djvused'
+
+DJVUSED_PATH = djvused_path
 
 def _djvused_usability_check():
     try:
@@ -46,7 +55,7 @@ class StreamEditor(object):
         self._file_name = file_name
         self._commands = []
         self._autosave = autosave
-    
+
     def clone(self):
         return StreamEditor(self._filename, self._autosave)
 
@@ -58,25 +67,25 @@ class StreamEditor(object):
 
     def select_all(self):
         self._add('select')
-    
+
     def select(self, page_id):
         self._add('select %s' % page_id)
-    
+
     def select_shared_annotations(self):
         self._add('select-shared-ant')
-    
+
     def create_shared_annotations(self):
         self._add('create-shared-ant')
-    
+
     def set_annotations(self, annotations):
         self._add('set-ant')
         for annotation in annotations:
             self._add(str(annotation))
         self._add('.')
-    
+
     def remove_annotations(self):
         self._add('remove-ant')
-    
+
     def print_annotations(self):
         self._add('print-ant')
 
@@ -89,7 +98,7 @@ class StreamEditor(object):
 
     def remove_metadata(self):
         self._add('remove-meta')
-    
+
     def set_text(self, text):
         if text is None:
             self.remove_text()
@@ -98,7 +107,7 @@ class StreamEditor(object):
 
     def remove_text(self):
         self._add('remove-txt')
-    
+
     def set_outline(self, outline):
         if outline is None:
             outline = ''
@@ -109,22 +118,22 @@ class StreamEditor(object):
 
     def remove_thumbnails(self):
         self._add('remove-thumbnails')
-    
+
     def set_page_title(self, title):
         self._add('set-page-title %s' % Expression(title))
-    
+
     def save_page(self, file_name, include = False):
         command = 'save-page'
         if include:
             command += '-with'
         self._add('%s %s' % command, file_name)
-    
+
     def save_as_bundled(self, file_name):
         self._add('save-bundled %s' % file_name)
 
     def save_as_indirect(self, file_name):
         self._add('save-indirect %s' % file_name)
-    
+
     def save(self):
         self._add('save')
 
@@ -150,7 +159,7 @@ class StreamEditor(object):
             raise IOError(djvused.stderr.readline().lstrip('* '))
         reader_thread.join()
         return result[0]
-        
+
     def commit(self):
         try:
             return self._execute(self._commands, save = self._autosave)
