@@ -1,5 +1,5 @@
 # encoding=UTF-8
-# Copyright © 2008, 2009, 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008, 2009, 2010, 2011, 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
 # Copyright © 2009 Mateusz Turcza <mturcza@mimuw.edu.pl>
 #
 # This package is free software; you can redistribute it and/or modify
@@ -18,10 +18,10 @@ LICENSE = 'GPL-2'
 
 import sys
 import itertools
+import functools
 import locale
 import os.path
 import threading
-import tempfile
 from Queue import Queue, Empty as QueueEmpty
 
 import djvusmooth.dependencies as __dependencies
@@ -42,6 +42,7 @@ from djvusmooth.gui.flatten_text import FlattenTextDialog
 from djvusmooth.gui.text_browser import TextBrowser
 from djvusmooth.gui.outline_browser import OutlineBrowser
 from djvusmooth.gui.maparea_browser import MapAreaBrowser
+from djvusmooth.gui.history import FileHistory
 from djvusmooth.gui import dialogs
 from djvusmooth.text import mangle as text_mangle
 import djvusmooth.models.metadata
@@ -56,7 +57,6 @@ from djvusmooth import __version__, __author__
 from djvusmooth.i18n import _
 
 MENU_ICON_SIZE = (16, 16)
-DJVU_WILDCARD = _('DjVu files (*.djvu, *.djv)|*.djvu;*.djv|All files|*')
 
 WxDjVuMessage, wx.EVT_DJVU_MESSAGE = wx.lib.newevent.NewEvent()
 
@@ -68,8 +68,17 @@ if wx.__version__.startswith('2.8.'):
 
 class OpenDialog(wx.FileDialog):
 
+    __wildcard = _(
+        'DjVu files (*.djvu, *.djv)|*.djvu;*.djv|'
+        'All files|*'
+    )
+
     def __init__(self, parent):
-        wx.FileDialog.__init__(self, parent, style = wx.OPEN, wildcard=DJVU_WILDCARD, message = _('Open a DjVu document'))
+        wx.FileDialog.__init__(self, parent,
+            style=wx.OPEN,
+            wildcard=self.__wildcard,
+            message=_('Open a DjVu document')
+        )
 
 class TextModel(models.text.Text):
 
@@ -262,16 +271,18 @@ class ScrolledPanel(wx.lib.scrolledpanel.ScrolledPanel):
         # So just skip the event:
         event.Skip()
 
-class MainWindow(wx.Frame):
+def skip_if_being_deleted(method):
+    @functools.wraps(method)
+    def method_wrapper(self, *args, **kwargs):
+        if self.IsBeingDeleted():
+            # This method could be called via an event while the window is
+            # being deleted. Do *not* fiddle with menu in such a case, as it
+            # would provoke segmentation fault.
+            return
+        return method(self, *args, **kwargs)
+    return method_wrapper
 
-    def _menu_item(self, menu, caption, help, method, style = wx.ITEM_NORMAL, icon = None, id = wx.ID_ANY):
-        item = wx.MenuItem(menu, id, caption, help, style)
-        if icon is not None:
-            bitmap = wx.ArtProvider_GetBitmap(icon, wx.ART_MENU, MENU_ICON_SIZE)
-            item.SetBitmap(bitmap)
-        self.Bind(wx.EVT_MENU, method, item)
-        menu.AppendItem(item)
-        return item
+class MainWindow(wx.Frame):
 
     @apply
     def default_xywh():
@@ -310,6 +321,14 @@ class MainWindow(wx.Frame):
             self._config['external_editor'] = value or ''
         return property(get, set)
 
+    @apply
+    def default_open_dir():
+        def get(self):
+            return self._config.read('open_dir', '')
+        def set(self, value):
+            self._config['open_dir'] = value or ''
+        return property(get, set)
+
     def save_defaults(self):
         self._config.flush()
 
@@ -322,6 +341,9 @@ class MainWindow(wx.Frame):
 
     def __init__(self):
         self._config = wx.GetApp().config
+        # The ._config attribute should not be accessed directly, but only
+        # via .default_* properties.
+        # Use .save_defaults() to save the config file.
         x, y, w, h = self.default_xywh
         wx.Frame.__init__(self, None, pos=(x, y), size=(w, h))
         self.setup_external_editor()
@@ -361,12 +383,13 @@ class MainWindow(wx.Frame):
         sizer.Add(self.page_widget, 0, wx.ALL, 0)
         self.editable_menu_items = []
         self.saveable_menu_items = []
-        self._setup_menu()
+        self.file_history = FileHistory(self._config)
+        self.create_menus()
         self.dirty = False
         self.do_open(None)
         self.Bind(wx.EVT_CLOSE, self.on_exit)
 
-    def _setup_menu(self):
+    def create_menus(self):
         menu_bar = wx.MenuBar()
         menu_bar.Append(self._create_file_menu(), _('&File'))
         menu_bar.Append(self._create_edit_menu(), _('&Edit'))
@@ -376,40 +399,58 @@ class MainWindow(wx.Frame):
         menu_bar.Append(self._create_help_menu(), _('&Help'));
         self.SetMenuBar(menu_bar)
 
+    def _create_menu_item(self, menu, caption, help, method, style=wx.ITEM_NORMAL, icon=None, id=wx.ID_ANY):
+        item = wx.MenuItem(menu, id, caption, help, style)
+        if icon is not None:
+            bitmap = wx.ArtProvider_GetBitmap(icon, wx.ART_MENU, MENU_ICON_SIZE)
+            item.SetBitmap(bitmap)
+        self.Bind(wx.EVT_MENU, method, item)
+        menu.AppendItem(item)
+        return item
+
     def _create_file_menu(self):
         menu = wx.Menu()
-        self._menu_item(menu, _('&Open') + '\tCtrl+O', _('Open a DjVu document'), self.on_open, icon=wx.ART_FILE_OPEN)
-        save_menu_item = self._menu_item(menu, _('&Save') + '\tCtrl+S', _('Save the document'), self.on_save, icon=wx.ART_FILE_SAVE)
-        close_menu_item = self._menu_item(menu, _('&Close') + '\tCtrl+W', _('Close the document'), self.on_close, id=wx.ID_CLOSE)
+        menu_item = functools.partial(self._create_menu_item, menu)
+        menu_item(_('&Open') + '\tCtrl+O', _('Open a DjVu document'), self.on_open, icon=wx.ART_FILE_OPEN)
+        recent_menu = wx.Menu()
+        recent_menu_item = menu.AppendMenu(wx.ID_ANY, _('Open &recent'), recent_menu)
+        self.file_history.set_menu(self, recent_menu_item, self.do_open)
+        save_menu_item = menu_item(_('&Save') + '\tCtrl+S', _('Save the document'), self.on_save, icon=wx.ART_FILE_SAVE)
+        close_menu_item = menu_item(_('&Close') + '\tCtrl+W', _('Close the document'), self.on_close, id=wx.ID_CLOSE)
         self.editable_menu_items += close_menu_item,
         self.saveable_menu_items += save_menu_item,
         menu.AppendSeparator()
-        self._menu_item(menu, _('&Quit') + '\tCtrl+Q', _('Quit the application'), self.on_exit, icon=wx.ART_QUIT)
+        menu_item(_('&Quit') + '\tCtrl+Q', _('Quit the application'), self.on_exit, icon=wx.ART_QUIT)
         return menu
 
     def _create_edit_menu(self):
         menu = wx.Menu()
-        self._menu_item(menu, _('&Metadata') + '\tCtrl+M', _('Edit the document or page metadata'), self.on_edit_metadata)
+        menu_item = functools.partial(self._create_menu_item, menu)
+        menu_item(_('&Metadata') + '\tCtrl+M', _('Edit the document or page metadata'), self.on_edit_metadata)
         submenu = wx.Menu()
-        self._menu_item(submenu, _('&External editor') + '\tCtrl+T', _('Edit page text in an external editor'), self.on_external_edit_text)
-        self._menu_item(submenu, _('&Flatten'), _('Remove details from page text'), self.on_flatten_text)
+        submenu_item = functools.partial(self._create_menu_item, submenu)
+        submenu_item(_('&External editor') + '\tCtrl+T', _('Edit page text in an external editor'), self.on_external_edit_text)
+        submenu_item(_('&Flatten'), _('Remove details from page text'), self.on_flatten_text)
         menu.AppendMenu(wx.ID_ANY, _('&Text'), submenu)
         submenu = wx.Menu()
-        self._menu_item(submenu, _('&Bookmark this page') + '\tCtrl+B', _('Add the current to document outline'), self.on_bookmark_current_page)
-        self._menu_item(submenu, _('&External editor'), _('Edit document outline in an external editor'), self.on_external_edit_outline)
-        self._menu_item(submenu, _('&Remove'), _('Remove document outline'), self.on_remove_outline)
+        submenu_item = functools.partial(self._create_menu_item, submenu)
+        submenu_item(_('&Bookmark this page') + '\tCtrl+B', _('Add the current to document outline'), self.on_bookmark_current_page)
+        submenu_item(_('&External editor'), _('Edit document outline in an external editor'), self.on_external_edit_outline)
+        submenu_item(_('&Remove all'), _('Remove whole document outline'), self.on_remove_outline)
         menu.AppendMenu(wx.ID_ANY, _('&Outline'), submenu)
         return menu
 
     def _create_view_menu(self):
         menu = wx.Menu()
+        menu_item = functools.partial(self._create_menu_item, menu)
         submenu = wx.Menu()
+        submenu_item = functools.partial(self._create_menu_item, submenu)
         for caption, help, method, id in \
         [
-            (_('Zoom &in'),  _('Increase the magnification'), self.on_zoom_in, wx.ID_ZOOM_IN),
-            (_('Zoom &out'), _('Decrease the magnification'), self.on_zoom_out, wx.ID_ZOOM_OUT),
+            (_('Zoom &in') + '\tCtrl++', _('Increase the magnification'), self.on_zoom_in, wx.ID_ZOOM_IN),
+            (_('Zoom &out') + '\tCtrl+-', _('Decrease the magnification'), self.on_zoom_out, wx.ID_ZOOM_OUT),
         ]:
-            self._menu_item(submenu, caption, help, method, id = id or wx.ID_ANY)
+            submenu_item(caption, help, method, id=id)
         submenu.AppendSeparator()
         for caption, help, zoom, id in \
         [
@@ -418,12 +459,12 @@ class MainWindow(wx.Frame):
             (_('&Stretch'),    _('Stretch the image to the window size'), StretchZoom(),  None),
             (_('One &to one'), _('Set full resolution magnification.'),   OneToOneZoom(), wx.ID_ZOOM_100),
         ]:
-            self._menu_item(submenu, caption, help, self.on_zoom(zoom), style=wx.ITEM_RADIO, id = id or wx.ID_ANY)
+            id = id or wx.ID_ANY
+            submenu_item(caption, help, self.on_zoom(zoom), style=wx.ITEM_RADIO, id=id)
         submenu.AppendSeparator()
         self.zoom_menu_items = {}
         for percent in 300, 200, 150, 100, 75, 50, 25:
-            item = self._menu_item(
-                submenu,
+            item = submenu_item(
                 '%d%%' % percent,
                 _('Magnify %d%%') % percent,
                 self.on_zoom(PercentZoom(percent)),
@@ -434,6 +475,7 @@ class MainWindow(wx.Frame):
             self.zoom_menu_items[percent] = item
         menu.AppendMenu(wx.ID_ANY, _('&Zoom'), submenu)
         submenu = wx.Menu()
+        submenu_item = functools.partial(self._create_menu_item, submenu)
         for caption, help, method in \
         [
             (_('&Color') + '\tAlt+C', _('Display everything'),                                            self.on_display_everything),
@@ -442,9 +484,10 @@ class MainWindow(wx.Frame):
             (_('&Background'),        _('Display only the background layer'),                             self.on_display_background),
             (_('&None') + '\tAlt+N',  _('Neither display the foreground layer nor the background layer'), self.on_display_none)
         ]:
-            self._menu_item(submenu, caption, help, method, style=wx.ITEM_RADIO)
+            submenu_item(caption, help, method, style=wx.ITEM_RADIO)
         menu.AppendMenu(wx.ID_ANY, _('&Image'), submenu)
         submenu = wx.Menu()
+        submenu_item = functools.partial(self._create_menu_item, submenu)
         _tmp_items = []
         for caption, help, method in \
         [
@@ -452,16 +495,17 @@ class MainWindow(wx.Frame):
             (_('&Hyperlinks') + '\tAlt+H', _('Display overprinted annotations'), self.on_display_maparea),
             (_('&Text') + '\tAlt+T',       _('Display the text layer'),          self.on_display_text),
         ]:
-            _tmp_items += self._menu_item(submenu, caption, help, method, style=wx.ITEM_RADIO),
-        self._menu_item_display_no_nonraster, self._menu_item_display_maparea, self._menu_item_display_text = _tmp_items
+            _tmp_items += [submenu_item(caption, help, method, style=wx.ITEM_RADIO)]
+        self.menu_item_display_no_nonraster, self.menu_item_display_maparea, self.menu_item_display_text = _tmp_items
         del _tmp_items
-        self._menu_item_display_no_nonraster.Check()
+        self.menu_item_display_no_nonraster.Check()
         menu.AppendMenu(wx.ID_ANY, _('&Non-raster data'), submenu)
-        self._menu_item(menu, _('&Refresh') + '\tCtrl+L', _('Refresh the window'), self.on_refresh)
+        menu_item(_('&Refresh') + '\tCtrl+L', _('Refresh the window'), self.on_refresh)
         return menu
 
     def _create_go_menu(self):
         menu = wx.Menu()
+        menu_item = functools.partial(self._create_menu_item, menu)
         for caption, help, method, icon in \
         [
             (_('&First page') + '\tCtrl-Home', _('Jump to first document page'),    self.on_first_page,    None),
@@ -470,20 +514,22 @@ class MainWindow(wx.Frame):
             (_('&Last page') + '\tCtrl-End',   _('Jump to last document page'),     self.on_last_page,     None),
             (_(u'&Go to page…') + '\tCtrl-G',  _(u'Jump to page…'),                 self.on_goto_page,     None)
         ]:
-            self._menu_item(menu, caption, help, method, icon = icon)
+            menu_item(caption, help, method, icon=icon)
         return menu
 
     def _create_settings_menu(self):
         menu = wx.Menu()
-        sidebar_menu_item = self._menu_item(menu, _('Show &sidebar') + '\tF9', _('Show/hide the sidebar'), self.on_show_sidebar, style=wx.ITEM_CHECK)
+        menu_item = functools.partial(self._create_menu_item, menu)
+        sidebar_menu_item = menu_item(_('Show &sidebar') + '\tF9', _('Show/hide the sidebar'), self.on_show_sidebar, style=wx.ITEM_CHECK)
         if self.default_sidebar_shown:
             sidebar_menu_item.Check()
-        self._menu_item(menu, _(u'External editor…'), _('Setup an external editor'), self.on_setup_external_editor)
+        menu_item(_(u'External editor…'), _('Setup an external editor'), self.on_setup_external_editor)
         return menu
 
     def _create_help_menu(self):
         menu = wx.Menu()
-        self._menu_item(menu, _('&About') + '\tF1', _('More information about this program'), self.on_about, id=wx.ID_ABOUT)
+        menu_item = functools.partial(self._create_menu_item, menu)
+        menu_item(_('&About') + '\tF1', _('More information about this program'), self.on_about, id=wx.ID_ABOUT)
         return menu
 
     def on_setup_external_editor(self, event):
@@ -548,10 +594,9 @@ class MainWindow(wx.Frame):
 
     def on_open(self, event):
         dialog = OpenDialog(self)
-        dialog.SetDirectory(self._config.read('open_dir', ''))
+        dialog.SetDirectory(self.default_open_dir)
         try:
             if dialog.ShowModal() == wx.ID_OK:
-                self._config['open_dir'] = os.path.dirname(dialog.GetPath()) or ''
                 self.do_open(dialog.GetPath())
         finally:
             dialog.Destroy()
@@ -644,17 +689,20 @@ class MainWindow(wx.Frame):
     def on_display_none(self, event):
         self.page_widget.render_mode = None
 
+    @skip_if_being_deleted
     def on_display_text(self, event):
         self.page_widget.render_nonraster = RENDER_NONRASTER_TEXT
-        self._menu_item_display_text.Check()
+        self.menu_item_display_text.Check()
 
+    @skip_if_being_deleted
     def on_display_maparea(self, event):
         self.page_widget.render_nonraster = RENDER_NONRASTER_MAPAREA
-        self._menu_item_display_maparea.Check()
+        self.menu_item_display_maparea.Check()
 
+    @skip_if_being_deleted
     def on_display_no_nonraster(self, event):
         self.page_widget.render_nonraster = None
-        self._menu_item_display_no_nonraster.Check()
+        self.menu_item_display_no_nonraster.Check()
 
     def on_refresh(self, event):
         self.Refresh()
@@ -865,7 +913,9 @@ class MainWindow(wx.Frame):
             try:
                 rc = dialog.ShowModal()
                 if rc == wx.ID_YES:
-                    return self.do_save()
+                    if not self.do_save():
+                        return False
+                    assert not self.dirty
                 elif rc == wx.ID_NO:
                     pass
                 elif rc == wx.ID_CANCEL:
@@ -882,6 +932,8 @@ class MainWindow(wx.Frame):
         if path is None:
             clear_models()
         else:
+            self.file_history.add(path)
+            self.default_open_dir = os.path.dirname(path)
             try:
                 self.document = self.context.new_document(djvu.decode.FileURI(path))
                 self.metadata_model = MetadataModel(self.document)
@@ -935,8 +987,17 @@ class MainWindow(wx.Frame):
         self.SetTitle(title)
 
     def on_about(self, event):
-        message = '%(APPLICATION_NAME)s %(__version__)s\n' + _('Author') + ': %(__author__)s\n' + _('License') + ': %(LICENSE)s'
-        message = message % globals()
+        message = (
+            '%(app)s %(version)s\n' +
+            _('Author') + ': %(author)s\n' +
+            _('License') + ': %(license)s'
+        )
+        message = message % dict(
+            app=APPLICATION_NAME,
+            version=__version__,
+            author=__author__,
+            license=LICENSE
+        )
         wx.MessageBox(message = message, caption = _(u'About…'))
 
     def handle_message(self, event):
@@ -997,7 +1058,9 @@ class Application(wx.App):
         window = MainWindow()
         window.Show(True)
         if argv:
-            window.do_open(argv[0])
+            path = argv[0]
+            path = os.path.abspath(path)
+            window.do_open(path)
         return self.MainLoop()
 
 # vim:ts=4 sw=4 et
