@@ -11,11 +11,11 @@
 # General Public License for more details.
 
 import re
-import exceptions
+import functools
 import warnings
 import weakref
 
-class NotOverriddenWarning(exceptions.UserWarning):
+class NotOverriddenWarning(UserWarning):
     pass
 
 def not_overridden(f):
@@ -29,18 +29,18 @@ def not_overridden(f):
     >>> B().f(6, 7)
     Traceback (most recent call last):
     ...
-    NotOverriddenWarning: `lib.varietes.B.f()` is not overriden
+    NotOverriddenWarning: `lib.varietes.B.f()` is not overridden
     >>> C().f(6, 7)
     42
     '''
+    @functools.wraps(f)
     def new_f(self, *args, **kwargs):
         cls = type(self)
         warnings.warn(
-            '`%s.%s.%s()` is not overriden' % (cls.__module__, cls.__name__, f.__name__),
+            '`%s.%s.%s()` is not overridden' % (cls.__module__, cls.__name__, f.__name__),
             category = NotOverriddenWarning,
             stacklevel = 2
         )
-
         return f(self, *args, **kwargs)
     return new_f
 
@@ -165,7 +165,7 @@ class idict(object):
 
     def __init__(self, **kwargs):
         self.__dict__.update(kwargs)
-    
+
     def __repr__(self):
         return '%s.%s(%s)' % \
         (
